@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# Copyright (c) 2016-2019 Sullivan Beck. All rights reserved.
+# Copyright (c) 2016-2021 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -7,6 +7,7 @@ use warnings;
 use strict;
 no strict 'subs';
 no strict 'refs';
+$| = 1;
 
 my %type = ('country'  => 'Country',
             'language' => 'Language',
@@ -16,65 +17,54 @@ my %type = ('country'  => 'Country',
             'langext'  => 'LangExt',
             'langvar'  => 'LangVar',
            );
-my $generic_tests;
 
-sub do_tests {
-   my($data_type,$inp_file,$test_type,$codeset,$show_errs) = @_;
-   my $type = $type{$data_type};
-   $::data_type = $data_type;
-   $::test_type = $test_type;
-   $inp_file    = $data_type  if (! $inp_file);
+sub init_tests {
+   # $0 = "DATATYPE_FLAG_NUM.t"
+   #       DATATYPE  = country, language, etc.
+   #       FLAG      = func, old, oo, or some other value
+   #
+   # If FLAG is func, old, or oo, then:
+   #     INPUTFILE is based on DATATYPE
+   #     TESTTYPE is the same as FLAG
+   # Otherwise
+   #     INPUTFILE is FLAG
+   #     TESTTYPE is oo
+   #
+   # Module is based on DATATYPE
 
-   my($runtests) = shift(@ARGV);
+   my $tmp      = $0;
+   $tmp         =~ s,^.*/,,;
+   $tmp         =~ s,\.t$,,;
+   my($dt,$flag,$n) = split(/_/,$tmp);
 
-   # Load the test function and the data for the tests
-
-   my($dir,$tdir);
-   if ( -f "t/testfunc.pl" ) {
-     require "./t/testfunc.pl";
-     require "./t/vals_${inp_file}.pl";
-     $dir="./lib";
-     $tdir="t";
-   } elsif ( -f "testfunc.pl" ) {
-     require "./testfunc.pl";
-     require "./vals_${inp_file}.pl";
-     $dir="../lib";
-     $tdir=".";
+   $::data_type = $dt;
+   my $inp_file;
+   if ($flag =~ /^(func|old|oo)$/) {
+      $::test_type = $flag;
+      $inp_file    = $dt;
    } else {
-     die "ERROR: cannot find testfunc.pl\n";
+      $::test_type = 'oo';
+      $inp_file    = $flag;
    }
+   my $mod         = $type{$::data_type};
 
-   unshift(@INC,$dir);
+   require "vals_${inp_file}.pl";
 
-   $::tests .= $generic_tests  if (! defined($show_errs));
-
-   if ($test_type eq 'old') {
-      $::module = "Locale::$type";
+   if ($::test_type eq 'old') {
+      $::module = "Locale::$mod";
       eval("use $::module");
       my $tmp   = $::module . "::show_errors";
-      &{ $tmp }(0);
-   } elsif ($test_type eq 'func') {
-      $::module = "Locale::Codes::$type";
+      &{ $tmp }(1);
+   } elsif ($::test_type eq 'func') {
+      $::module = "Locale::Codes::$mod";
       eval("use $::module");
       my $tmp   = $::module . "::show_errors";
-      &{ $tmp }(0);
-   } elsif (defined($codeset)) {
+      &{ $tmp }(1);
+   } else {
       eval("use Locale::Codes");
-      $::obj = Locale::Codes->new($data_type,$codeset,$show_errs);
+      $::obj = new Locale::Codes $::data_type;
       $::obj->show_errors(1);
-   } elsif (defined($show_errs)) {
-      eval("use Locale::Codes");
-      $::obj = Locale::Codes->new();
-      $::obj->type($data_type);
-      $::obj->show_errors($show_errs);
-   } else {
-      eval("use Locale::Codes");
-      $::obj = new Locale::Codes $data_type;
-      $::obj->show_errors(0);
    }
-
-   print "$::data_type [$::test_type]\n";
-   test_Func(\&test,$::tests,$runtests);
 }
 
 sub test {
@@ -89,7 +79,7 @@ sub test {
    }
 
    if ($stderr) {
-      $stderr =~ s/\n.*//m;
+      $stderr =~ s/\n.*//s;
       chomp($stderr);
       return $stderr;
    } else {
@@ -226,135 +216,10 @@ sub _test {
    }
 }
 
-$generic_tests = "
-#################
-
-2code
-_undef_
-   _undef_
-
-2code
-   _undef_
-
-2code
-_blank_
-   _undef_
-
-2code
-UnusedName
-   _undef_
-
-2code
-   _undef_
-
-2code
-_undef_
-   _undef_
-
-2name
-_undef
-   _undef_
-
-2name
-   _undef_
-
-###
-
-add
-AAA
-newCode
-   1
-
-2code
-newCode
-   aaa
-
-delete
-AAA
-   1
-
-2code
-newCode
-   _undef_
-
-###
-
-add
-AAA
-newCode
-   1
-
-rename
-AAA
-newCode2
-   1
-
-2code
-newCode
-   aaa
-
-2code
-newCode2
-   aaa
-
-###
-
-add_alias
-newCode2
-newAlias
-   1
-
-2code
-newAlias
-   aaa
-
-delete_alias
-newAlias
-   1
-
-2code
-newAlias
-   _undef_
-
-###
-
-replace_code
-AAA
-BBB
-   1
-
-2name
-AAA
-   newCode2
-
-2name
-BBB
-   newCode2
-
-###
-
-add_code_alias
-BBB
-CCC
-   1
-
-2name
-BBB
-   newCode2
-
-2name
-CCC
-   newCode2
-
-delete_code_alias
-CCC
-   1
-
-2name
-CCC
-   _undef_
-
-";
+init_tests();
+$::t->tests(func  => \&test,
+            tests => $::tests);
+$::t->done_testing();
 
 1;
 # Local Variables:
